#!/usr/bin/env python
# 2007-01-21
import wx
import sys, os, os.path, datetime, re

from wxremindrc import *

def alert(sound, display, message):
	global audible1, audible2, visible2, alert_sound, alert_display
	if sound == None: 
		alert_sound = int(alert_sound)
	else:
		alert_sound = int(sound)
	if display == None: 
		alert_display = int(alert_display)
	else:
		alert_display = int(display)
	event_msg = spoken_msg = message
	if alert_sound == 0 and alert_display == 0:
		sys.exit(0)

	if alert_sound == 1:
		os.system(audible1)

	elif alert_sound == 2:
		if event_msg and alert_parsenums:
			# look for three consecutive digits preceeded by a non-digit and
			# followed by either a word boundary or a non-digit
			regex= re.compile(r'\D?(\d{3,3})[\b\D]')
			nums = re.findall(regex, spoken_msg)
			if len(nums) > 0:
				for num in nums:
					d0 = num[0]
					d1 = num[1]
					d2 = num[2]
					if d1 == '0':
						# replace the zero with an uppercase o
						d1 = 'O'
					# separate the leading digit from the following 2 digits
					rep = " %s %s%s" % (d0,d1,d2)
					# replace the original number
					spoken_msg = re.sub(num, rep, spoken_msg)
				spoken_msg.strip()

		hours,minutes,seconds = \
				datetime.datetime.today().strftime("%H %M %S").split()

		hours = int(hours)
		minutes = int(minutes)
		seconds = int(seconds)

		if twelvehour:
			oclock = " o'clock"
			min = ""
			mins = ""
			spokenhours = ("twelve", "one", "two", "three", "four", "five",
			"six", "seven", "eight", "nine", "ten", "eleven", "twelve",
			"one", "two", "three", "four", "five", "six", "seven", "eight",
			"nine", "ten", "eleven")
		else:
			oclock = ""
			min = " minute"
			mins = " minutes"
			spokenhours = ("zero hours", "one hour", "two hours", "three hours",
					"four hours", "five hours", "six hours", "seven hours",
					"eight hours", "nine hours", "ten hours", "eleven hours",
					"twelve hours", "thirteen hours", "fourteen hours", 
					"fifteen hours", "sixteen hours", "seventeen hours",
					"eighteen hours", "nineteen hours", "twenty hours",
					"twenty one hours", "twenty two hours", "twenty three hours")

		# round current time up
		if seconds >= 50:
			minutes += 1
			if minutes >= 60:
				minutes = 0
				hours += 1
				if hours >= 24:
					hours = 0

		message = ""

		if alert_greeting:
			try:
				Who = " %s" % alert_whom
			except:
				Who = ''
			if hours < 12:
				message = "Good morning%s. " % Who
			elif hours < 18:
				message = "Good afternoon%s. " % Who
			else:
				message = "Good evening, %s. " % Who

		time = spokenhours[hours]

		if minutes == 0:
			time += oclock
		elif minutes == 1:
			if twelvehour:
				time += " O 1%s" % min
			else:
				time += " 1%s" % min
		elif minutes <= 9:
			if twelvehour:
				# replace the leading zero with an upper case o
				minutes = "O %d" % minutes 
			time += " %s%s" % (minutes, mins)
		else:
			time += " %s%s" % (minutes, mins)


		message += "The time is %s. %s" % (time, spoken_msg)

		os.system(audible2 % message)

	if alert_display == 1:
		# show a dialog box
		app = wx.PySimpleApp()
		dlg = wx.MessageDialog(None, "%s" % event_msg,
							  'wxRemind alert',
							  wx.OK | wx.ICON_INFORMATION)
		dlg.ShowModal()
		dlg.Destroy()

	elif alert_display == 2:
		# call growlnotify
		os.system(visible2 % event_msg)

def main():
	from optparse import OptionParser
	import os.path
	global options, args
	parser = OptionParser(version = "%s" % version,
		usage = "%prog [options] message",
		description = """Produce an audible and/or visible alert using settings from ~/.wxremindrc.  Designed to be called from remind's 'RUN'.""")
	parser.add_option("-d", "--display", action = "store", type="choice", 
		dest='display', choices = ["0", "1", "2"], default = None,
		help = """0: none; 1: message box; 2: visable2""")
	parser.add_option("-s", "--sound", action = "store", type="choice", 
		dest='sound', choices = ["0", "1", "2"], default = None,
		help = """0: none; 1: audible1; 2: audible2""")
	
	(options, args) = parser.parse_args()
	if len(args):
		message = ' '.join(args)
	else:
		message = ''
	alert(options.sound, options.display, message)
