#!/usr/bin/python

import os.path, sys, datetime, re, commands

from wxremindrc import *

class RemData:

	leadingzero = re.compile(r'^0')

	nummonths = 4
	if twelvehour:
		timefmt = "%I:%M%p"
		indent1 = 9
		indent2 = 17
	else:
		indent1 = 7
		indent2 = 13
		timefmt = "%H:%M"

	datefmt = "%a %d %b %Y"
	regat = re.compile(r'\s+at.*$')
	data = {}
	searchstr = ''
	startdate = ''
	foundlist = []
	lastfound = ''
	nextitem = 0
	oneday = datetime.timedelta(days=1)
	
	# Since remind only recognizes English month abbreviations but python gives
	# the appropriate Locale version.
	mnthabbrv = {
	'01' : 'Jan',
	'02' : 'Feb',
	'03' : 'Mar',
	'04' : 'Apr',
	'05' : 'May',
	'06' : 'Jun',
	'07' : 'Jul',
	'08' : 'Aug',
	'09' : 'Sep',
	'10' : 'Oct',
	'11' : 'Nov',
	'12' : 'Dec'
	}

	def _init__(self):
		self.getMonths()

	def getMonths(self):
		self.data = {}
		today = datetime.date.today()
		year, month = map(int, today.strftime("%Y %m").split())
		if month == 1:
			startmonth = 12
			startyear = year - 1
		else:
			startmonth = month - 1
			startyear = year
		self.slurp(startyear, startmonth, self.nummonths)

	def getDay(self, y, m, d):
		year, month, day = map(int, (y,m,d))
		requesteddate = datetime.date(year,month,day)
		self.shortremdate = requesteddate.strftime("%d %m")
		if not self.data.has_key(year) or not self.data[year].has_key(month):
			self.slurp(y,m,1)
		try:
			retval = self.data[year][month][day]
		except:
			retval = None
		return retval

	def getOccurances(self):
		command = "%s -n %s | sort" % \
				(remind, reminders)
		events = commands.getoutput(command).split('\n')
		nevents = []
		for event in events:
			parts = event.split()
			year, month, day = map(int, parts[0].split('/'))
			parts[0] = datetime.date(year, month, day).strftime(self.datefmt)
			nevents.append(" ".join(parts))
		res = "\n".join(nevents)
		return res

	def getMonthlyDurations(self, y, m):
		year, month = map(int, (y, m))
		busy = {}
		if not self.data.has_key(year) or not self.data[year].has_key(month):
			self.slurp(y,m,1)
		if self.data.has_key(year) and self.data[year].has_key(month):
			for day in self.data[year][month].keys():
				minutes = 0
				for event in self.data[year][month][day]:
					minutes += event[6]
				busy.setdefault(day, minutes)
		return busy

	def slurp(self, y, m, n):
		startyear, startmonth, months = map(int, (y, m, n))
		startmonth = "%02d" % startmonth
		# Slurp remind output for the relevant date and number of months
		command = "%s -b2 -rls%s %s %s %s" % (remind, months, reminders, 
			self.mnthabbrv[startmonth], startyear)
		lines = commands.getoutput(command).split('\n')
		linenum = 0 
		filename = ''
		for line in lines:
			parts = line.split()
			if len(parts) == 0:
				continue
			if parts[1] == "fileinfo":
				linenum, filename = parts[2:4]
				# go to the next line for the item details
				continue
			if parts[0] == "REMIND":
				msg = """
Fatal error: the command:
%s
raised a syntax error message from remind.""" % command
				raise msg
			try:
				year, month, day = map(int, parts[0].split('/'))
			except:
				msg = "Error parsing remind output: %s" % line
				raise msg
			date = datetime.date(year, month, day).strftime(self.datefmt)
			durationminutes, startminutes = parts[3:5]
			if startminutes != "*":
				startminutes = int(startminutes)
				starttime = datetime.datetime(year,month,day,
						startminutes/60,startminutes%60)
				if durationminutes != "*":
					durationminutes = int(durationminutes)
					durationdelta = datetime.timedelta(hours=durationminutes/60,
							minutes=durationminutes%60)
					end = starttime + durationdelta
					endtime = end.strftime(self.timefmt)
					if twelvehour:
						endtime = self.leadingzero.sub(' ', endtime)
				else:
					endtime = ''
					durationminutes = zerominutes
				starttime = starttime.strftime(self.timefmt)
				if twelvehour:
					starttime = self.leadingzero.sub(' ', starttime)
			else:
				starttime = ''
				endtime = ''
				durationminutes = zerominutes
			if starttime and endtime:
				interval = "-"
			else:
				interval = ""
			msg = " ".join(parts[5:])

			# The following creates a value for data[year][month][day] creating
			# year, month and day keys when necessary.	Is this cool or what?
			self.data.setdefault(year,{}).setdefault(month,{}).setdefault(day,
					[]).append([starttime,	# 0
						interval,			# 1
						endtime,			# 2
						msg,				# 3
						filename,			# 4
						linenum,			# 5
						durationminutes,	# 6
						])

	def firstOccurance(self, str):
		if not self.startdate:
			self.startdate = datetime.date.today().strftime("%d %m %Y")
		self.nextitem = 0
		self.foundlist = []
		self.searchstr = str
		d,m,y = self.startdate.split()
		self.lastfound = (y,m,d)
		mnth = self.mnthabbrv["%02d" % int(m)]
		command = "%s -b2 -n %s %s %s %s | grep -i \'%s\' | sort" % \
				(remind, reminders, d, mnth, y, self.searchstr)
		events = commands.getoutput(command).split('\n')
		for event in events:
			if len(event) > 0:
				parts = event.split()
				year, month, day = map(int, parts[0].split('/'))
				msg = "%s" % self.regat.sub('', " ".join(parts[1:]))
				self.foundlist.append((year, month, day, msg))
		return self.nextOccurance()

	def nextOccurance(self):
		numfound = len(self.foundlist)
		if numfound > 0 and self.nextitem < numfound:
			retval = self.foundlist[self.nextitem] 
			self.lastfound = retval[:-1]
			self.nextitem += 1
			return retval
		else:
			return None

def main():
	global options, args, days
	from optparse import OptionParser
	parser = OptionParser(version = "%s" % version,
		description = """Send nicely formatted output from remind to standard output.""")
	parser.add_option("-b", "--begin", action = "store", type="string", dest='begin',
		default = datetime.date.today().strftime("%Y-%m-%d"),
		help = """starting date: YYYY-MM-DD, Defaults to today""")
	parser.add_option("-d", "--days", action = "store", type="string", dest='days', 
		default="%d" % days, help = """number of days to display/search, defaults to %d""" % days)
	parser.add_option("-s", "--search", action = "store", type="string",
		dest='search', default="", help = """search for string within DAYS""")
	parser.add_option("-f", "--fileinfo", action = "store_true", default=False, 
		dest="fileinfo", help = """append file name and line number information""")
	parser.add_option("-n", "--nextoccurances", action = "store_true",
		default=False, dest='next', 
		help = """list next occurances (ignores other options)""")
	
	(options, args) = parser.parse_args()
	(y,m,d) = map(int, options.begin.split('-'))
	date = datetime.date(y, m, d)
	days = int(options.days)
	mydata = RemData()
	count = 0
	if options.next:
		print mydata.getOccurances()
	else:
		if options.search:
			regs = re.compile(r'%s' % options.search, re.IGNORECASE)
		else:
			regs = re.compile(r'.*')
		while count < days:
			y, m, d = date.strftime("%Y %m %d").split()
			count += 1
			daystr = "%s:\n" % date.strftime("%a %d %b %Y")
			if mydata.getDay(y,m,d) != None:
				found = False
				for event in mydata.getDay(y,m,d):
					str = " ".join(event[:-3])
					if not options.search or regs.search(str):
						found = True
						file = event[4]
						line = event[5]
						if options.fileinfo:
							daystr += "    %s [%s:%s]\n" % (str.lstrip(), file, line)
						else:
							daystr += "    %s\n" % str
				if found:
					print "%s" % daystr 
			date = date + RemData.oneday
