#!/usr/bin/env python
# 2007-01-25 1:55 PM
import os, os.path, shutil
import wx
import wx.calendar
import datetime
import linecache
import wx.lib.analogclock as ac 
import wx.lib.mixins.listctrl  as  listmix

from wxRemHelp import Help
from wxRemAbout import About
from wxRemHints import Hints
from wxRemDays import Days
import wxRemEdit
import textwrap

twl = textwrap.TextWrapper(width=116)

from wxremindrc import *

if ec_border == -1:
	BORDER = wx.SUNKEN_BORDER
elif ec_border == 0:
	BORDER = wx.NO_BORDER
elif ec_border == 1:
	BORDER = wx.RAISED_BORDER
elif ec_border == 2:
	BORDER = wx.SIMPLE_BORDER

# set the border styles
if sb_border == -1:
	SBORDER = wx.SB_NORMAL
elif sb_border == 0:
	SBORDER = wx.SB_FLAT
elif sb_border == 1:
	SBORDER = wx.SB_RAISED
elif sb_border == 2:
	SBORDER = wx.SIMPLE_BORDER

sb_string = "Press F1 for information about wxRemind or ? for help."

# Prepare the replacment hash for later updating.
rephash = {'e' : editor, 'f' : None, 'n' : None, 
	'y' : None, 'm' : None, 'd' : None}

import wxremdata
Data = wxremdata.RemData()

class MyClock(ac.AnalogClock):
	# Keep the clock from taking the focus
	def AcceptsFocus(*args, **kwargs):
		return False

class MyDateBar(wx.StaticText):
	# Keep the clock from taking the focus
	def AcceptsFocus(*args, **kwargs):
		return False

class MyDetailBar(wx.StaticText):
	# Keep the clock from taking the focus
	def AcceptsFocus(*args, **kwargs):
		return False

class MyListCtrl(wx.ListView, listmix.ListCtrlAutoWidthMixin):
	def __init__(self, parent, ID, pos=wx.DefaultPosition,
				 size=wx.DefaultSize, style=0):
		wx.ListCtrl.__init__(self, parent, ID, pos, size, style)
		listmix.ListCtrlAutoWidthMixin.__init__(self)
	def AcceptsFocus(*args, **kwargs):
		return True

class MyCalCtrl(wx.calendar.CalendarCtrl):
	# Let the calendar take the focus
	def AcceptsFocus(*args, **kwargs):
		return True

class MyFrame(wx.Frame):
	def __init__(self):
		self.data = []
		self.calattr = []
		self.currentItem = 0
		self.currentCount = 0
		self.calfocus = 0
		fontfam = wx.DEFAULT
		bfont = wx.Font(basefontsize + buttonfontadj, fontfam, 
				wx.NORMAL, wx.NORMAL)
		lfont = wx.Font(basefontsize + listfontadj, fontfam, 
				wx.NORMAL, wx.NORMAL)
		dfont = wx.Font(basefontsize + datefontadj, fontfam,
				wx.NORMAL, wx.NORMAL)
		sfont = wx.Font(basefontsize + statusfontadj, fontfam,
				wx.NORMAL, wx.NORMAL)
		cfont = wx.Font(basefontsize + calendarfontadj, wx.DEFAULT,
				wx.NORMAL, wx.NORMAL)
		wx.Frame.__init__(self, None, -1, 'wxRemind')
		self.frame_id = self.GetId()

		# The top date bar
		self.datebar = MyDateBar(self, -1, "Test Date Bar", size=(-1,-1),
				style = wx.NO_BORDER | wx.ST_NO_AUTORESIZE | wx.TE_CENTRE )
		self.datebar.SetFont(dfont)
		self.datebar_id = self.datebar.GetId()

		# The bottom (selected event details) bar
		self.detailbar = MyDetailBar(self, -1, "", size=(-1,-1),
				style = wx.NO_BORDER | wx.ST_NO_AUTORESIZE)
		self.detailbar.SetFont(sfont)
		self.detailbar_id = self.detailbar.GetId()

		# The today button
		self.tdy = wx.Button(self, -1, '', style = wx.SB_FLAT)
		self.tdy.SetFont(bfont)
		self.Bind(wx.EVT_BUTTON, self.OnClick, self.tdy)
		self.tdy.Bind(wx.EVT_CHAR, self.OnChar)
		self.tdy.SetBackgroundColour(bgcolor)
		self.tdy.SetToolTipString("Show events for today.")
		self.tdy_id = self.tdy.GetId()

		# The event list
		self.lc = MyListCtrl(self, -1, size=eventsize,
				style=wx.LC_REPORT | wx.LC_SINGLE_SEL
				| wx.LC_NO_HEADER | wx.WANTS_CHARS
				| BORDER )
		self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.lc) 
		self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated, self.lc) 
		self.lc.Bind(wx.EVT_CHAR, self.OnChar)
		self.lc.SetFont(lfont)
		self.lc.InsertColumn(0, '00:00AM')
		self.lc.InsertColumn(1, '-')
		self.lc.InsertColumn(2, '00:00PM')
		self.lc.InsertColumn(3, 'Message')
		self.lc.SetColumnWidth(0, -2)
		self.lc.SetColumnWidth(1, -2)
		self.lc.SetColumnWidth(2, -2)
		self.lc.SetColumnWidth(3, -1)
		self.lc_id = self.lc.GetId()

		# The calendar
		self.cal = MyCalCtrl(self, -1, wx.DateTime_Now(), 
				style = wx.calendar.CAL_SHOW_HOLIDAYS 
				| wx.calendar.CAL_SEQUENTIAL_MONTH_SELECTION
				| BORDER 
				| wx.WANTS_CHARS 
				)
		self.cal.SetHeaderColours(headercolor,fcolor)
		self.cal.SetHolidayColours(holidaycolor,fcolor)
		self.Bind(wx.calendar.EVT_CALENDAR_SEL_CHANGED, self.OnCalSelected, 
				id=self.cal.GetId())
		self.cal.Bind(wx.EVT_SET_FOCUS, self.OnSetFocus)
		self.cal.Bind(wx.EVT_CHAR, self.OnChar)
		self.cal_id = self.cal.GetId()

		# The analog clock
		self.clk = MyClock(self, size=clocksize, 
				hoursStyle=ac.TICKS_CIRCLE, 
				clockStyle=ac.SHOW_HOURS_TICKS | 
				ac.SHOW_HOURS_HAND | ac.SHOW_MINUTES_HAND | 
				ac.SHOW_SECONDS_HAND )
		self.clk.SetTickSize(16)
		self.clk.SetTickFillColour(handcolor)
		self.clk.SetTickBorderColour('BLACK')
		self.clk.SetHandFillColour(handcolor)
		self.clk.SetHandBorderColour('BLACK')
		self.clk.SetFaceBorderWidth(0)
		self.clk.SetFaceBorderColour(clockcolor)
		self.clk.SetBackgroundColour(clockcolor)
		self.clk.SetFaceFillColour(clockcolor)
		self.clk_id = self.clk.GetId()


		# Key bindings and focus
		self.clk.Bind(wx.EVT_CHAR, self.OnChar)
		self.clk.Bind(wx.EVT_MOUSE_EVENTS, self.Focus('res'))
		self.detailbar.Bind(wx.EVT_CHAR, self.OnChar)
		self.detailbar.Bind(wx.EVT_MOUSE_EVENTS, self.Focus('res'))


		# The layout
		vbox = wx.BoxSizer(wx.VERTICAL)
		hbox1  = wx.BoxSizer(wx.HORIZONTAL)
		vbox1 = wx.BoxSizer(wx.VERTICAL)
		vbox2 = wx.BoxSizer(wx.VERTICAL)
		vbox1.Add(self.datebar, 0, wx.EXPAND 
			| wx.LEFT 
			| wx.RIGHT 
			| wx.TOP | wx.BOTTOM, 8)
		vbox1.Add(self.lc, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 2)
		vbox2.Add(self.tdy, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | 
				wx.TOP | wx.RIGHT | wx.LEFT | wx.BOTTOM, 4)
		vbox2.Add(self.clk, -1, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL |
				wx.TOP | wx.RIGHT | wx.LEFT | wx.BOTTOM, 4)
		vbox2.Add(self.cal, -1, wx.EXPAND 
				| wx.ALIGN_CENTER 
				| wx.TOP | wx.RIGHT | wx.LEFT, 4
				)
		hbox1.Add(vbox1, 0, wx.EXPAND)
		hbox1.Add(vbox2, 0, wx.EXPAND)
		hbox2 = wx.BoxSizer(wx.HORIZONTAL)
		vbox3 = wx.BoxSizer(wx.VERTICAL)
		vbox3.Add(self.detailbar, -1, wx.EXPAND | wx.ALL, 2)
		hbox2.Add(vbox3, -1, wx.EXPAND)
		vbox.Add(hbox1, -1, wx.EXPAND)
		vbox.Add(hbox2, 0, wx.EXPAND)
		self.SetSizerAndFit(vbox)
		width, height = self.GetSizeTuple()
		self.SetSizeHints(width, height, width, height)

		# Show events for the current date.
		self.today = None
		self.Today()
		self.Focus()

	def FormatDate(self,y,m,d):
		y,m,d = map(int, (y,m,d))
		date = datetime.date(y,m,d)
		day = int(date.strftime("%d"))
		fmtstr = "%%a, %d %%b %%Y" % day
		return date.strftime(fmtstr)

	def OnClick(self, event):
		self.Today()

	def Today(self):
		today = datetime.date.today()
		if not self.today or self.today != today:
			# reset data to show 'current date only' events today
			Data.getMonths()
		self.today = today
		y,m,d = self.today.strftime("%Y %m %d").split()
		self.selday = (y,m,d)
		self.tdy.SetLabel("%s " % self.FormatDate(y,m,d))
		self.cal.SetDate(wx.DateTime_Now())
		self.showDay(self.selday[0], self.selday[1], self.selday[2])

	def OnMonthChange(self, y, m):
		y, m = map(int, (y,m))
		busy = Data.getMonthlyDurations(y,m)
		for i in range(1,32):
			self.cal.ResetAttr(i)
		for day in busy.keys():
			minutes = busy[day]
			attr = wx.calendar.CalendarDateAttr()
			if minutes < busy0[0]:
				attr.SetTextColour('BLACK')
				self.cal.SetAttr(day, attr)
			elif minutes < busy1[0]:
				attr.SetTextColour(busy0[1])
				self.cal.SetAttr(day, attr)
			elif minutes < busy2[0]:
				attr.SetTextColour(busy1[1])
				self.cal.SetAttr(day, attr)
			elif minutes < busy3[0]:
				attr.SetTextColour(busy2[1])
				self.cal.SetAttr(day, attr)
			elif minutes < busy4[0]:
				attr.SetTextColour(busy3[1])
				self.cal.SetAttr(day, attr)
			else:
				attr.SetTextColour(busy4[1])
				self.cal.SetAttr(day, attr)

	def OnCalSelected(self, event):
		y,m,d = event.GetDate().Format("%Y %m %d").split()
		self.showDay(y, m, d)

	def OnItemActivated(self, event):
		# Open the relevant file at the appropriate line number for editing.
		# only call editor if we have both a file and a line number 
		if self.data[self.currentItem][3]:
			rephash['f'] = self.data[self.currentItem][4]
			rephash['n'] = self.data[self.currentItem][5]
			if editor == '':
				self.infoMessage("The internal editor is not an option in this version.\nCorrect the setting in ~/.wxremindrc to use an external editor.", 
				'Error', wx.ICON_ERROR)
			else:
				editlist = (editold % rephash).split()
				retval = os.spawnv(os.P_WAIT, editor, editlist)
				if retval > 0:
					self.infoMessage(
			'Attempt to start editor %s failed. Check setting in ~/.wxremindrc.' \
					% editor, 'Error')
				else:
					self.Refresh()

	def OnItemSelected(self, event):
		# Show the relevant reminder in the status bar.
		self.currentItem = event.m_itemIndex
		if self.data[self.currentItem][4]:
			file = self.data[self.currentItem][4]
			line = int(self.data[self.currentItem][5])
			reminder = linecache.getline(file,line).strip()
			if len(reminder) > 116:
				reminder = twl.wrap(reminder)[0] + " ..."
			self.detailbar.SetLabel(reminder)
			self.detailbar.Wrap(-1)
		self.Focus('lc')
		self.lc.EnsureVisible(self.currentItem)
		self.lc.SetItemState(self.currentItem, 
			wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED, 
			wx.LIST_STATE_SELECTED )
		event.Skip()

	def OnItemDeSelected(self, event):
		item = event.GetItem()

	def OnAbout(self, event):
		# show the about page
		About(self)

	def OnHelp(self, event):
		# show the help page
		dlg = Help(self)
		dlg.ShowModal()
		dlg.Destroy()

	def OnHints(self, event):
		# show the hints page
		dlg = Hints(self)
		dlg.ShowModal()
		dlg.Destroy()

	def OnDays(self):
		y,m,d = datetime.date(self.selday[0], self.selday[1],
			self.selday[2]).strftime("%Y %m %d").split()
		dlg = Days(self, y,m,d)
		dlg.ShowModal()
		dlg.Destroy()

	def OnMonth(self):
		# show the postscript calendar for the month
		date = datetime.date(self.selday[0],
				self.selday[1],self.selday[2]).strftime("%b %Y")
		makecal = "%s -p %s %s | %s" \
				% (remind, reminders, date, rem2ps)
		os.system("%s && %s" % (makecal, viewps))

	def OnSetFocus(self, event):
		# called by EVT_SET_FOCUS. sets bgcolor to match focus
		w = wx.Window_FindFocus()
		if w:
			curr_id = w.GetId()
			if curr_id == self.cal_id:
				self.Focus('cal')
			elif curr_id == self.lc_id:
				self.Focus('lc')
		event.Skip()

	def Focus(self, w=''):
		# Called with an argument sets appropriate focus, else toggles focus.
		if w == 'cal':
			self.setFocus('cal')
		elif w == 'lc':
			self.setFocus('lc')
		elif w == 'res':
			if self.calfocus:
				self.setFocus('cal')
			else:
				self.setFocus('lc')
		else: 
			if self.calfocus:
				self.setFocus('lc')
			else:
				self.setFocus('cal')

	def setFocus(self, target):
		if target == 'cal':
			self.cal.SetFocus()
			self.calfocus = 1
			self.detailbar.SetLabel(sb_string)
			self.cal.SetHeaderColours(headercolor,fcolor)
			self.cal.SetBackgroundColour(fcolor)
			self.lc.SetBackgroundColour(nfcolor)
			for day in range(1,32):
				attr = self.cal.GetAttr(day)
				if attr and attr.HasTextColour():
					tc = attr.GetTextColour()
					attr = wx.calendar.CalendarDateAttr()
					attr.SetTextColour(tc)
					attr.SetBackgroundColour(fcolor)
					self.cal.SetAttr(day, attr)
				else:
					attr = wx.calendar.CalendarDateAttr()
					attr.SetBackgroundColour(fcolor)
					self.cal.SetAttr(day,attr)
		elif target == 'lc':
			self.lc.SetFocus()
			self.lc.SetBackgroundColour(fcolor)			
			self.calfocus = 0
			self.cal.SetHeaderColours(headercolor,nfcolor)
			self.cal.SetBackgroundColour(nfcolor)
			for day in range(1,32):
				attr = self.cal.GetAttr(day)
				if attr and attr.HasTextColour():
					tc = attr.GetTextColour()
					attr = wx.calendar.CalendarDateAttr()
					attr.SetTextColour(tc)
					attr.SetBackgroundColour(nfcolor)
					self.cal.SetAttr(day, attr)
				else:
					attr = wx.calendar.CalendarDateAttr()
					attr.SetBackgroundColour(nfcolor)
					self.cal.SetAttr(day,attr)

	def showDay(self,y,m,d):
		# Show the events for the specified day in the event list.
		y, m, d= map(int, (y,m,d))
		self.data = Data.getDay(y,m,d)
		if m != self.selday[1] or y != self.selday[0]:
			# the month has changed
			self.OnMonthChange(y,m)
		self.selday = (y,m,d) 
		date = datetime.date(y,m,d)
		self.lc.DeleteAllItems()
		if self.data:
			for i in range(len(self.data)):
				index = self.lc.InsertStringItem(i, self.data[i][0])
				self.lc.SetStringItem(index, 0, self.data[i][0])
				self.lc.SetStringItem(index, 1, self.data[i][1])
				self.lc.SetStringItem(index, 2, self.data[i][2])
				self.lc.SetStringItem(index, 3, self.data[i][3])
				self.lc.SetItemData(index, i)
		self.datebar.SetLabel(self.FormatDate(y,m,d))
		if date == datetime.date.today():
			self.datebar.SetForegroundColour(todaycolor)
		else:
			self.datebar.SetForegroundColour("BLACK")
		self.currentCount = self.lc.GetItemCount()
		self.Focus('cal')
		self.detailbar.SetLabel(sb_string)

	def OnChar(self, event):
		curr_id = self.FindFocus().GetId()
		keycode = event.GetKeyCode()
		if keycode == 9:			# tab
			self.Focus()
		elif keycode == 17:			# Ctrl-Q quit
			self.OnQuit()
		elif keycode == 32:			# space
			self.Today()
		elif keycode == 47:			# /: search
			self.OnSearch(event)
		elif keycode == 63:			# ? Show Help box
			self.OnHelp(event)
		elif keycode == 82:			# R refresh
			self.Refresh()
		elif keycode == 97:			# a: alert event
			self.newEvent('a')
		elif keycode == 99:			# c show html calendar for month 
			self.OnMonth()
		elif keycode == 100:		# d show agenda for the next 'days'  
			self.OnDays()
		elif keycode == 101:		# e: edit any event
			self.Edit(event)
		elif keycode == 102:		# f: floating reminder
			self.newEvent('f')
		elif keycode == 104:		# h: show hints
			self.OnHints(event)
		elif keycode == 110:		# n: search next
			self.findNext(event)
		elif keycode == 116:		# t (timed event)
			self.newEvent('t')
		elif keycode == 117:		# u untimed event
			self.newEvent('u')
		elif keycode == wx.WXK_F1:	# F1 Show About box
			self.OnAbout(event)
		elif curr_id == self.lc_id:
			count = self.currentCount
			index = self.currentItem
			if keycode == 13:       # activate
				self.lc.Connect(-1,-1,
					wx.wxEVT_COMMAND_LIST_ITEM_ACTIVATED,
					self.OnItemActivated(index))
			if keycode == wx.WXK_UP and count:		# previous event
				new = max(0, index-1)
				self.lc.EnsureVisible(new)
				self.lc.SetItemState(new, 
					wx.LIST_STATE_SELECTED, 
					wx.LIST_STATE_SELECTED )
			elif keycode == wx.WXK_DOWN and count:		# next event
				new = min(count-1, index+1)
				self.lc.EnsureVisible(new)
				self.lc.SetItemState(new, 
					wx.LIST_STATE_SELECTED, 
					wx.LIST_STATE_SELECTED )
		else:
			event.Skip()

	def newEvent(self, type):
		# Create a new event and append it to the reminders file
		# count changes made to data before opening dialog
		changes = 0 
		y,m,d = map(int, self.selday)
		date = datetime.date(y, m, d)
		datefmt = "%d %s" % (d, date.strftime("%b %Y"))
		data = { "date" : datefmt }
		changes += 1
		if type == 'a':
			if alert_sound in (0,1,2):
				# this change doesn't affect the relevant count
				data["alert_s"] = int(alert_sound)
			if alert_display in (0,1,2):
				# this change doesn't affect the relevant count
				data["alert_d"] = int(alert_display)
			if alert_other_message:
				data["omsg"] = alert_other_message
				changes += 1

		dlg = wxRemEdit.MyDialog(data, type)
		dlg.UnsavedChanges -= changes
		dlg.ShowModal()
		dlg.Destroy()
		if data.has_key('msg') and data['msg'] != '':
			# user did not press cancel
			if data.has_key('dur') and data['dur']:
				data['dur'] = "DUR %s" % data['dur']
			else:
				data['dur'] = ''
			if data.has_key('omsg') and data['omsg']:
				data['msg'] = '%%"%s%%" %s' % (data['msg'], data['omsg'])
			else:
				data['msg'] = '%%"%s%%"' % data['msg']
			if type == 'a':
				# convert radio button index to switch string 
				data['alert'] = '-d%s -s%s' % (data['alert_d'], data['alert_s'])
				rem = 'REM %(date)s AT %(time)s %(dur)s RUN wxremalert ' % data
				rem += '%(alert)s %(msg)s%%' % data
			elif type == 'u':
				rem = 'REM %(date)s MSG %(msg)s%%' % data
			elif type == 't':
				rem = 'REM %(date)s AT %(time)s %(dur)s MSG %(msg)s%%' % data
			elif type == 'f':
				d,m,y = data['date'].split()
				data['date'] = "[float(%s,%s,%s,%s)]" % (y,m,d, data['warn'])
				rem = 'REM %(date)s MSG %(msg)s%%' % data
			if rem:
				rephash['f'] = "%s" % reminders
				# Backup the existing reminders file
				shutil.copyfile(reminders, "%s.bak" % reminders)
				remread = open(reminders,'r')
				remlines = remread.readlines()
				remread.close()
				# make sure the last line ends with a \n
				lastrem = remlines.pop().rstrip()
				lastrem = "%s\n" % lastrem
				remlines.append(lastrem)
				# append the new reminder
				remlines.append(rem)
				remwrite = open(reminders, 'w')
				remwrite.writelines(remlines)
				remwrite.close()
				self.Refresh()

	def Edit(self,evt):
		# Open the default reminders file at the last line for editing.
		rephash['f'] = "%s" % reminders
		rephash['n'] = -1
		if editor == '':
			self.infoMessage("The internal editor is not an option in this version.\nCorrect the setting in ~/.wxremindrc to use an external editor.", 
			'Error', wx.ICON_ERROR)
		else:
			editlist = (editnew % rephash).split()
			retval = os.spawnv(os.P_WAIT, editor, editlist)
			if retval > 0:
				self.infoMessage("Unable to edit using:\n    '%s'\nCheck setting in ~/.wxremindrc." % editor, 
				'Error', wx.ICON_ERROR)
			else:
				self.Refresh() 

	def Refresh(self):
		Data.getMonths()
		linecache.clearcache()
		self.OnMonthChange(self.selday[0], self.selday[1])
		self.showDay(self.selday[0], self.selday[1], self.selday[2])

	def OnSearch(self,evt):
		y,m,d = map(int, self.selday)
		startingdate = datetime.date(y,m,d).strftime("%a, %d %b %Y")
		Data.startdate = "%s %s %s" % (d, m, y)
		dlg = wx.TextEntryDialog(
				self, 'Please enter case-insensitive search string:', 
				'Searching from %s' % startingdate, '')
		dlg.SetBackgroundColour(nfcolor)
		dlg.SetValue("")
		if dlg.ShowModal() == wx.ID_OK:
			str = dlg.GetValue()
			day = Data.firstOccurance(str)
			if day:
				self.showDay(day[0],int(day[1]),int(day[2]))
				self.cal.SetDate(wx.DateTimeFromDMY(int(day[2]), int(day[1])-1,int(day[0])))
			else:
				self.notFound()
		dlg.Destroy()

	def findNext(self,evt):
		if Data.nextitem and Data.searchstr:
			day = Data.nextOccurance()
			if day:
				self.showDay(int(day[0]),int(day[1]),int(day[2]))
				self.cal.SetDate(wx.DateTimeFromDMY(int(day[2]),int(day[1])-1,int(day[0])))
			else:
				self.notFound()
		else:
			self.OnSearch(evt)

	def notFound(self):
		year,month,day = Data.lastfound
		date = self.FormatDate(year, month, day)
		self.infoMessage("No occurances of '%s' were found after %s." % 
				(Data.searchstr, date),
				'Failed Search', wx.ICON_INFORMATION)

	def infoMessage(self, message, title, icon):
		dlg = wx.MessageBox("%s" % message, 
			caption = "%s" % title, style = wx.OK | icon)

	def OnQuit(self):
		self.Destroy()

class App(wx.App):
	def OnInit(self):
		import wx.lib.colourdb
		wx.lib.colourdb.updateColourDB()
		self.frame = MyFrame()
		self.frame.Show()
		self.SetTopWindow(self.frame)
		return True

def main():
	app = App()
	app.MainLoop()	
