#!/usr/bin/python
import os.path
has_wxremindrc = False


version = "0.8.5"		# the current version of wxremind
cf_version = (0,1)		# the current version of this configuration file
rc_version = (0,0)		# this will be updated to the version stored in
						# ~/.wxremindrc 

search_path = os.getenv('PATH').split(os.pathsep)
wxremindrc = os.path.expanduser('~/.wxremindrc')
wxremfloat = os.path.expanduser("~/.wxremfloat")
reminders = os.path.expanduser("~/.reminders")
msg = []
error = []
warn = []
rc = []

def PathSearch(filename):
	for path in search_path:
		candidate = os.path.join(path,filename)
		if os.path.os.path.isfile(candidate):
			return os.path.abspath(candidate)
	return ''
	

def make_wxremindrc():
	import sys, os, shutil
	global msg, error, warn, rc
		
	rc.append("""# wxRemind configuration
# Documentation for configuration options is available at 
# <http://www.duke.edu/~dgraham/wxRemind>.""")
	rc.append("rc_version = (%s,%s)" % cf_version)
	
	if sys.platform == 'darwin':
		mac = True
	else:
		mac = False
	
	calendars = os.path.expanduser("~")
	rc.append("calendars = '%s'" % calendars)
	
	remind = PathSearch('remind')
	if remind: 
		has_remind = True
		rc.append("remind = '%s'" % remind)
	else: 
		has_remind = False
		rc.append("remind = ''")
		error.append("Could not find remind (required for everything).")
	
	rem2ps = PathSearch('rem2ps')
	if rem2ps: 
		has_rem2ps = True
		rem2ps = '%s -l > %s/.calendar.ps' % (rem2ps, calendars)
		rc.append("rem2ps = '%s'" % rem2ps)
	else: 
		has_rem2ps = False
		warn.append("Could not find rem2ps (required for monthly calendar).")
		rc.append("rem2ps = ''")
	
	if mac: 
		viewps = 'open -a Preview %s/.calendar.ps' % calendars
	else:
		gv = PathSearch('gv')
		if gv:
			viewps = '%s --orientation=landscape --media=LETTER %s/calendar.ps' % (gv, calendars)
		else:
			viewps = ''
	rc.append("viewps = '%s'" % viewps)
	rc.append("wxremindrc = '%s'" % wxremindrc)
	days = 7
	rc.append("days = %s" % days)
	
	twelvehour = 1
	rc.append("twelvehour = %s" % twelvehour)
	
	if mac:
		editor = PathSearch('edit')
		if editor:
			editold = '%(e)s +%(n)s -w --new-window %(f)s'
			editnew = '%(e)s +99999 --new-window %(f)s'
		else:
			editor = PathSearch('bbedit')
			if editor:
				editold = '%(e)s +%(n)s -w --new-window %(f)s'
				editnew = '%(e)s +99999 --new-window %(f)s'
			else: 
				editor = PathSearch('mate')
				if editor:
					editold = '%(e)s +%(n)s -w --new-window %(f)s'
					editnew = '%(e)s +99999 --new-window %(f)s'
				else:
					editor = ''
					editold = ''
					editnew = ''
					warn.append("""\
Could not find 'edit' (textwrangler), 'bbedit' or 'mate' (textmate).
To be able to edit your reminders files, you will either need to install
one of the above editors or to customize
	%s 
to use a text editor you have installed which can be called from the
command line and which will open a file at a specified line number.""")
	else: # not mac
		editor = PathSearch('gvim')
		if editor:
			editold = "%(e)s -f +%(n)s %(f)s"
			editnew = "%(e)s -f + %(f)s"
		else:
			editor = PathSearch('emacs')
			if editor:
				editold = "%(e)s +%(n)s %(f)s"
				editnew = "%(e)s +999999 %(f)s"
			else:
				editor = ''
				editold = ''
				editnew = ''
				warn.append("""\
Could not find either 'gvim' or 'emacs'. To be able to edit your reminders
files, you will either need to install one of the above editors or to
customize
	%s 
to use a text editor you have installed which can be called from the
command line and which will open a file at a specified line number.""")
	rc.append("editor = '%s'" % editor)			
	rc.append("editold = '%s'" % editold)			
	rc.append("editnew = '%s'" % editnew)			
	
	if mac:				
		audible1 = """osascript <<EOT
set soundFile to (alias "Macintosh HD:System:Library:Sounds:Basso.aiff")
tell application "Play Sound"
	play soundFile
end tell
EOT
"""
		audible2 = """osascript <<EOT
set this_voice to "Bruce"
set this_string to "%s"
say this_string using this_voice
EOT
"""
	else:
		audible1 = 'play /usr/share/sounds/KDE_Notify.wav'
		fest = PathSearch('festival')
		if fest:
			audible2 = "echo '%%s' | %s --tts" % fest
		else:
			audible2 = ''
	rc.append('audible1 = """%s"""' % audible1)			
	rc.append('audible2 = """%s"""' % audible2)			
	
	visible2 = ''
	if mac:
		growlnotify = PathSearch('growlnotify')
		if growlnotify:
			visible2 =	'%s -a ical -t wxRemind -m "%%s"' % growlnotify
	rc.append("visible2 = '%s'" % visible2)
	
	alert_sound = 1
	rc.append("alert_sound = %s" % alert_sound)
	alert_display = 1
	rc.append("alert_display = %s" % alert_display)
	alert_greeting = 1
	rc.append("alert_greeting = %s" % alert_greeting)
	alert_whom = ''
	rc.append("alert_whom = '%s'" % alert_whom)
	alert_parsenums = 1
	rc.append("alert_parsenums = %s" % alert_parsenums)
	alert_other_message = '%1'
	rc.append("alert_other_message = '%s'" % alert_other_message)
	
	if mac:
		basefontsize = 11
	else:
		basefontsize = 10		
	rc.append("basefontsize = %s" % basefontsize)
	statusfontadj = 0
	rc.append("statusfontadj = %s" % statusfontadj)
	# The adjustment to the base font size for the event list
	listfontadj = 0
	rc.append("listfontadj = %s" % listfontadj)
	datefontadj = 1
	rc.append("datefontadj = %s" % datefontadj)
	buttonfontadj = 0
	rc.append("buttonfontadj = %s" % buttonfontadj)
	calendarfontadj = 0
	rc.append("calendarfontadj = %s" % calendarfontadj)
	
	eventsize = (520,300)
	rc.append("eventsize = (%s, %s)" % eventsize)
	calsizeadj = (0, 0) 
	rc.append("calsizeadj = (%s, %s)" % calsizeadj)
	clocksize = (145,145)
	rc.append("clocksize = (%s, %s)" % clocksize)
	
	zerominutes = 15
	rc.append("zerominutes = %s" % zerominutes)
	
	busy0 = (30,'SLATEBLUE2')
	rc.append("busy0 = (%s, '%s')" % busy0)
	busy1 = (75, 'BLUE2')
	rc.append("busy1 = (%s, '%s')" % busy1)
	busy2 = (150, 'FORESTGREEN')
	rc.append("busy2 = (%s, '%s')" % busy2)
	busy3 = (225, 'BROWN')
	rc.append("busy3 = (%s, '%s')" % busy3)
	busy4 = (300, 'VIOLETRED')
	rc.append("busy4 = (%s, '%s')" % busy4)
	
	holidaycolor = 'BLACK'
	rc.append("holidaycolor = '%s'" % holidaycolor)
	headercolor = 'NAVYBLUE'
	rc.append("headercolor = '%s'" % headercolor)
	todaycolor = 'BLUE2'
	rc.append("todaycolor = '%s'" % todaycolor)
	
	if mac:
		clockcolor = "GRAY93"
	else:
		clockcolor = "GRAY90"
	rc.append("clockcolor = '%s'" % clockcolor)
	
	handcolor  = "GRAY20"
	rc.append("handcolor = '%s'" % handcolor)
	bgcolor	   = "GRAY99"
	rc.append("bgcolor = '%s'" % bgcolor)
	nfcolor	   = "GRAY95"
	rc.append("nfcolor = '%s'" % nfcolor)
	fcolor	   = "GRAY98"
	rc.append("fcolor = '%s'" % fcolor)
	
	ec_border = 2
	rc.append("ec_border = %s" % ec_border)
	sb_border = 0
	rc.append("sb_border = %s" % sb_border) 

	if len(error) == 0:
		if has_wxremindrc:
			shutil.copyfile(wxremindrc, "%s.bak" % wxremindrc)
			msg.append("%s has been updated. Created new file and backed up original." % wxremindrc)
		else:
			msg.append("Could not find %s. Created file with default values." % wxremindrc)
		fo = open(wxremindrc,'w')
		for line in rc:
			fo.write("%s\n" % line)
		fo.close()

def make_wxremfloat():	
	global msg, error, warn, rc
	rc.append("wxremfloat = '%s'" % wxremfloat)
	fo = open(wxremfloat, 'w')
	fo.write("""# Support for floating reminders
# This file should be named ~/.wxremfloat
# To enable floating reminders make sure your reminders file 
# includes the line
#
#	  INCLUDE %s
#
# prior to any use of float()
#
IF	($CalMode || $PSCal )
	# For Cal and PS calendars, only trigger the event on the due date.
	FSET float(y,m,d,n) trigger(date(y,m,d))
ELSE
	# For Simple Calendar (rem -s) and other modes, also set warning trigger,
	# priority and suffixes.
	FSET float(y,m,d,n) iif(date(y,m,d) == today(), trigger(today()) + \\
	" PRIORITY 1000", trigger(MAX(realtoday(), date(y,m,d)-n)) \\
	+ iif(realtoday() > date(y,m,d) - n, " PRIORITY " + (1000 + (date(y,m,d) \\
	- realtoday())), " PRIORITY " + (1000 + n)))

	FSET msgsuffix(x) iif(0 > x, " ", \\
	998 >= x, " (" + (1000-x) + " days ago)", \\
	999 >= x, " (yesterday)", \\
	1000 >= x, " (today)", \\
	1001 >= x, " (tomorrow)", \\
	2000 >= x, " (in " + (x - 1000) + " days)", \\
	x >= 2000, " ", \\
	" ")

	# This calsuffix is used by wxRemind
	FSET calsuffix(x) iif(0 > x, " ", \\
	998 >= x, " (" + (1000-x) + " days ago)", \\
	999 >= x, " (yesterday)", \\
	1000 >= x, " (today)", \\
	1001 >= x, " (tomorrow)", \\
	2000 >= x, " (in " + (x - 1000) + " days)", \\
	x >= 2000, " ", \\
	" ")
ENDIF
""" % wxremfloat)
	fo.close()
	msg.append("Could not find %s. Created file." % wxremfloat)

def make_reminders():
	global msg, error, warn, rc
	import shutil 
	rc.append("reminders = '%s'" % reminders)
	include_line = "INCLUDE %s\n" % wxremfloat
	if os.path.isfile(reminders): 
		has_reminders = True
		import re
		regex = re.compile(r'^\s*INCLUDE .*\.wxremfloat')
		fo = open(reminders, 'r')
		lines = fo.readlines()
		fo.close()
		found = False
		for line in lines:
			if regex.match(line):
				found = True
				break
		if not found:
			lines.insert(0, include_line)
			msg.append("Backed up %s and inserted line to include ~/.wxremfloat" % reminders)
		shutil.copyfile(reminders, "%s.bak" % reminders)
		fo = open(reminders, 'w')
		fo.writelines(lines)
		fo.close()		
	else: 
		has_reminders = False
		fo = open(reminders, 'w')
		fo.write(include_line)
		fo.close()
		msg.append("Could not find %s. Created file." % reminders)


if os.path.isfile(wxremindrc):
	execfile(wxremindrc)
	has_wxremindrc = True
else:
	has_wxremindrc = False

if not has_wxremindrc or rc_version < cf_version: 
	make_wxremindrc()
	execfile(wxremindrc)

if not os.path.isfile(wxremfloat) or rc_version < cf_version: 
	make_wxremfloat()

make_reminders()

if len(error) > 0:
	print "Fatal errors:"
	for line in error:
		print "	 %s" % line
if len(warn) > 0:
	print "Warnings:"
	for line in warn:
		print "	 %s" % line
if len(msg) > 0:
	msg.append("")
	msg.append("Please remember to send your comments to <daniel.graham@duke.edu>")
	msg.append("and to rate wxRemind at <http://freshmeat.net/projects/wxrem/>.")
	msg.append("")
	msg.append("			Thanks for using wxRemind!\n")
	print "Messages:"
	for line in msg:
		print "	 %s" % line
